//#  -*- mode: c++ -*-
//#
//#  TDSi2cdefs.h: i2c defintions for the TDS board.
//#
//#  Copyright (C) 2002-2004
//#  ASTRON (Netherlands Foundation for Research in Astronomy)
//#  P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
//#
//#  This program is free software; you can redistribute it and/or modify
//#  it under the terms of the GNU General Public License as published by
//#  the Free Software Foundation; either version 2 of the License, or
//#  (at your option) any later version.
//#
//#  This program is distributed in the hope that it will be useful,
//#  but WITHOUT ANY WARRANTY; without even the implied warranty of
//#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//#  GNU General Public License for more details.
//#
//#  You should have received a copy of the GNU General Public License
//#  along with this program; if not, write to the Free Software
//#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//#
//#  $Id$

#ifndef TDSI2CDEFS_H_
#define TDSI2CDEFS_H_

#include <Common/LofarTypes.h>

// initialize I2C expander 1, PLL control
// initialize I2C expander 1, Clock select/LED/Lock Detect
// initialize I2C expander 2, LED
// initialize I2C expander 2, revision number
#define TDS_INIT						\
      0x06, 0x42 >> 1, 0x06, 0x00, 0x07, 0x42 >> 1, 0x06,	\
      0x06, 0x42 >> 1, 0x02, 0x93, 0x07, 0x42 >> 1, 0x02,	\
      0x06, 0x42 >> 1, 0x07, 0xC0, 0x07, 0x42 >> 1, 0x07, 	\
      0x06, 0x42 >> 1, 0x03, 0x00, 0x07, 0x42 >> 1, 0x03,	\
      0x06, 0x44 >> 1, 0x07, 0x00, 0x07, 0x44 >> 1, 0x07,	\
      0x06, 0x44 >> 1, 0x03, 0x00, 0x07, 0x44 >> 1, 0x03,	\
      0x06, 0x44 >> 1, 0x06, 0xFF, 0x07, 0x44 >> 1, 0x06
#define TDS_INIT_SIZE 49

#define TDS_INIT_RESULT				\
      0x00, 0x00, 0x00, 0x00, 0x93, 0x00,	\
      0x00, 0xC0, 0x00, 0x00, 0x00, 0x00,	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	\
      0x00, 0xFF, 0x00
#define TDS_INIT_RESULT_SIZE 21

// read revision number
#define TDS_READ_REVISION			\
      0x07, 0x44 >> 1, 0x00
#define TDS_READ_REVISION_SIZE 3
#define TDS_READ_REVISION_RESULT_SIZE 2

// read lock detect
#define TDS_READ_LOCKDETECT			\
      0x07, 0x42 >> 1, 0x01
#define TDS_READ_LOCKDETECT_SIZE 3
#define TDS_READ_LOCKDETECT_RESULT_SIZE 2

// LED1 ON
#define TDS_LED1_ON						\
      0x06, 0x42 >> 1, 0x03, 0x08, 0x07, 0x42 >> 1, 0x03

#define TDS_LED1_ON_RESULT			\
      0x00, 0x08, 0x00

// LED1 OFF
#define TDS_LED1_OFF						\
      0x06, 0x42 >> 1, 0x03, 0x00, 0x07, 0x42 >> 1, 0x03

#define TDS_LED1_OFF_RESULT			\
      0x00, 0x00, 0x00

// LED2 ON
#define TDS_LED2_ON						\
      0x06, 0x44 >> 1, 0x03, 0x01, 0x07, 0x44 >> 1, 0x03

#define TDS_LED2_ON_RESULT			\
      0x00, 0x01, 0x00

// LED2 OFF
#define TDS_LED2_OFF						\
      0x06, 0x44 >> 1, 0x03, 0x00, 0x07, 0x44 >> 1, 0x03

#define TDS_LED2_OFF_RESULT			\
      0x00, 0x00, 0x00

#define TDS_LED_SIZE        7
#define TDS_LED_RESULT_SIZE 3

// i2c sequences to switch the VCXO's for 160MHZ and 200MHz on or off
#define TDS_VCXO_OFF \
      0x06, 0x42 >> 1, 0x02, 0x00, 0x07, 0x42 >> 1, 0x02
#define TDS_VCXO_OFF_SIZE 7

#define TDS_VCXO_OFF_RESULT \
      0x00, 0x00, 0x00
#define TDS_VCXO_OFF_RESULT_SIZE 3

#define TDS_VCXO_ON \
      0x06, 0x42 >> 1, 0x02, 0x03, 0x07, 0x42 >> 1, 0x02
#define TDS_VCXO_ON_SIZE 7

#define TDS_VCXO_ON_RESULT \
      0x00, 0x03, 0x00
#define TDS_VCXO_ON_RESULT_SIZE 3

// 160MHz to backplane
#define TDS_160MHZ							\
      0x06, 0x42 >> 1, 0x02, 0x01, 0x07, 0x42 >> 1, 0x02,		\
      0x06, 0x42 >> 1, 0x03, 0x01, 0x07, 0x42 >> 1, 0x03      
#define TDS_160MHZ_SIZE 14

#define TDS_160MHZ_RESULT			\
      0x00, 0x01, 0x00, 			\
      0x00, 0x01, 0x00
#define TDS_160MHZ_RESULT_SIZE 6

// 200MHz to backplane
#define TDS_200MHZ							\
      0x06, 0x42 >> 1, 0x02, 0x02, 0x07, 0x42 >> 1, 0x02,		\
      0x06, 0x42 >> 1, 0x03, 0x03, 0x07, 0x42 >> 1, 0x03
#define TDS_200MHZ_SIZE 14

#define TDS_200MHZ_RESULT			\
      0x00, 0x02, 0x00, 			\
      0x00, 0x03, 0x00
#define TDS_200MHZ_RESULT_SIZE 6

// switch off clock to backplane
#define TDS_OFF							\
      0x06, 0x42 >> 1, 0x02, 0x00, 0x07, 0x42 >> 1, 0x02
#define TDS_OFF_SIZE 7

#define TDS_OFF_RESULT				\
      0x00, 0x00, 0x00
#define TDS_OFF_RESULT_SIZE 3

//
// Initialization sequence to program the PLL's
//
// Generated with geni2c < TDS_program_plls.txt
// This generates two output files protocol.list (this list) and
// result.list
//
// Skipped read back to allow using smaller protocol_list register.
#define TDS_PROGRAMPLLS												\
      0x06, 0x42 >> 1, 0x02, 0x93, 0x06, 0x42 >> 1, 0x02, 0x93, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0xDB, 	\
      0x06, 0x42 >> 1, 0x02, 0xDB, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x6F, 	\
      0x06, 0x42 >> 1, 0x02, 0x4B, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x93, 0x06, 0x42 >> 1, 0x02, 0x93, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x93, 	\
      0x06, 0x42 >> 1, 0x02, 0x93, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x43, 	\
      0x06, 0x42 >> 1, 0x02, 0x67, 0x06, 0x42 >> 1, 0x02, 0x43, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x27, 0x06, 0x42 >> 1, 0x02, 0x03, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x27, 	\
      0x06, 0x42 >> 1, 0x02, 0x03, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0x6F, 0x06, 0x42 >> 1, 0x02, 0x4B, 	\
      0x06, 0x42 >> 1, 0x02, 0xDB 
#define TDS_PROGRAMPLLS_SIZE (301 * 4)

//
// Expected result of initialization sequence to program the PLL's
//
// Generated with geni2c < TDS_program_plls.txt
// This generates two output files protocol.list and result.list (this list).
//
#define TDS_PROGRAMPLLS_RESULT																\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 	\
      0x00, 0x00, 0x00, 0x00, 0x00 
#define TDS_PROGRAMPLLS_RESULT_SIZE (301 * 1)

#define TDS_C_END             0x13
#define TDS_C_END_SIZE        1
#define TDS_C_END_RESULT      0x00
#define TDS_C_END_RESULT_SIZE 1

#define TDS_C_WAIT             0x12, 0x00, 0x00, 0x04, 0xE1 // 1250 cycles * 8ns = 10 ms
#define TDS_C_WAIT_SIZE        5
#define TDS_C_WAIT_RESULT      0x00
#define TDS_C_WAIT_RESULT_SIZE 1

// Define bytesequences for reading the TDS board voltages
// write byte config; read byte 2v5; read byte 3v3 ; read byte temp 
#define TDS_READ_VOLT	\
		0x06, 0x14, 0x40, 0x11,	\
		0x07, 0x14, 0x20, \
		0x07, 0x14, 0x22, \
		0x07, 0x14, 0x27
#define TDS_READ_VOLT_SIZE	13

// write result ; 2v5 ; 3v3 ; temp
#define TDS_READ_VOLT_RESULT	\
		0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00
#define TDS_READ_VOLT_RESULT_SIZE	7

// Define bytesequences for reading the SPU board voltages
// write byte config; read byte 2v5 ; read byte 3v3; read byte 12v ; read byte vcc ; read byte temp 
#define TDS_READ_SPU	\
		0x06, 0x15, 0x40, 0x11,	\
		0x07, 0x15, 0x20, \
		0x07, 0x15, 0x22, \
		0x07, 0x15, 0x21, \
		0x07, 0x15, 0x23, \
		0x07, 0x15, 0x27
#define TDS_READ_SPU_SIZE	19

// write result ; 2v5 ; 3v3 ; 12v ; vcc ; temp
#define TDS_READ_SPU_RESULT	\
		0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00,	\
		0xAA, 0x00
#define TDS_READ_SPU_RESULT_SIZE	11

//
// Uncommand next line to prevent programming of PLL
// 
//#define DISABLE_PROGRAMPLL

namespace LOFAR {
  namespace RSP {


    extern uint8 tds_pll_result[ TDS_INIT_RESULT_SIZE
			                   + TDS_PROGRAMPLLS_RESULT_SIZE ];
                    
	extern uint8 tds_160MHz_result[ TDS_160MHZ_RESULT_SIZE
					              + TDS_C_END_RESULT_SIZE ];

	extern uint8 tds_200MHz_result[ TDS_200MHZ_RESULT_SIZE
					              + TDS_C_END_RESULT_SIZE ];

	extern uint8 tds_off_result[ TDS_VCXO_OFF_RESULT_SIZE
                               + TDS_OFF_RESULT_SIZE
                               + TDS_C_END_RESULT_SIZE ];

	extern uint8 tds_readstatus_result[ TDS_READ_LOCKDETECT_RESULT_SIZE
									  + TDS_READ_VOLT_RESULT_SIZE
									  + TDS_READ_SPU_RESULT_SIZE
									  + TDS_C_END_RESULT_SIZE ];
  };
};
     
#endif /* TDSI2CDEFS_H_ */

